program Puck_3D_sm
!
!Jesse's Puck driver - adapted from Morgan's Puck2

  use sep
  use puck_no_wind_smooth_mod

  implicit none
  
real, pointer, dimension(:,:,:)	:: d, p1, p2,px,py, residual, coh, residualx, cohx, residualy, cohy, a1
	real							:: dx, dt,dy,y0,x0,t0,mesh2phys
	integer							:: nt, nx, ny,nwx,nwt,Morgan, wk,boxsz
	integer							:: it,ix,iy
        complex , dimension(:,:,:)                                :: cp,dip

	!! hook up sep parameter input facility
	call sep_init(SOURCE)

	!! input parameters from history file
	call from_history("n1",nt); call from_history("n2",nx); call from_history("n3",ny)
	call from_history("d1",dt); call from_history("d2",dx); call from_history("d3",dy)
	call from_history("o1",t0); call from_history("o2",x0); call from_history("o3",y0)
      
	!! input window parameters from parameters
	call from_param("nwt",nwt,1); call from_param("nwx", nwx,1)
	call from_param("Morgan",Morgan,0);  call from_param("wk",wk,1)
        call from_param("boxsz",boxsz,0);
	!! output parameters to history file
	call to_history( "n1", nt);  call to_history( "n2", nx);  call to_history( "n3", ny)
	call to_history( "d1", dt);  call to_history( "d2", dx);  call to_history( "d3", dy)
	call to_history( "o1", t0);  call to_history( "o2", x0);  call to_history( "o3", y0)
	call to_history("esize",8);
	call to_history("Morgan",Morgan); call to_history("wk",wk)

	!! allocate data and dip arrays
	allocate(d(nt,nx,ny), p1(nt,nx,ny), p2(nt,nx,ny),px(nt,nx,ny),py(nt,nx,ny), residualx(nt,nx,ny),cohx(nt,nx,ny),residualy(nt,nx,ny),cohy(nt,nx,ny),cp(nt,nx,ny),a1(nt,nx,ny),dip(nt,nx,ny))

	!! read input data in and initialize dip arrays
	call sep_read(d)
     

	p1=0.0; p2=0.0

	write(0,*) 'nt/nx/ny:',nt,nx,ny
	write(0,*) 'min/max/avg(d)',minval(d),maxval(d),sum(d)/size(d)

	!! call subroutine to compute the dip fields
!	call puck2_init(nt, nx, d, nwt, nwx)
!	call puck2_exec(Morgan, wk, p1, p2, residual)

write(0,*) 'about to run puck in x dir'
do iy=1,ny  ! box sz is the size of the rectangularly box smoother...for a 1 2 1 tri-filter this will be 2.
     call puck_jesse_sm(d(:,:,iy),cohx(:,:,iy),px(:,:,iy),residualx(:,:,iy),boxsz)
end do

write(0,*) 'about to run puck in y dir'
do ix=1,nx
     call puck_jesse_sm(d(:,ix,:),cohy(:,ix,:),py(:,ix,:),residualy(:,ix,:),boxsz)
end do
write(0,*) 'finished puck in y dir'

!calculate dip 
p1=sqrt(px*px+py*py)
write(0,*) p1(50,50:60,50:70)

!calculate azimuth
a1=atan2(py,px)


!do ix=1,nx
!	do it=1,nt
!		write(0,*)'ix=',ix,' it=',it,' residual=',residual(it,ix)
!	end do
!end do
	!! output dip arrays to standard SEP history files
!	call sep_write(p1)


!	call sep_write(p2)

!make pick output
mesh2phys=dt/dx
cp=cmplx(cohx,p1*mesh2phys)
dip=cmplx(px,py)  !put x-dir gradient as real, and y-dir gradient as imag
 
!call sep_write(dip)  ! output complex for gradient in each direction            
call srite("out",dip,8*nt*nx*ny)
                call to_history("n1",nt,"picks"); call to_history("n2",nx,"picks")
                call to_history("n3",ny,"picks")
		call to_history("d1",dt,"picks"); call to_history("d2",dx,"picks")
                call to_history("d3",dy,"picks")
		call to_history("o1",t0,"picks"); call to_history("o2",x0,"picks")
                call to_history("o3",y0,"picks")
                call to_history("esize",8,"picks")
		call sep_write(cp,"picks")	



                call to_history("n1",nt,"abs_dip"); call to_history("n2",nx,"abs_dip")
                call to_history("n3",ny,"abs_dip")
		call to_history("d1",dt,"abs_dip"); call to_history("d2",dx,"abs_dip")
                call to_history("d3",dy,"abs_dip")
		call to_history("o1",t0,"abs_dip"); call to_history("o2",x0,"abs_dip")
                call to_history("o3",y0,"abs_dip")
                call to_history("esize",4,"abs_dip")
		call sep_write(p1,"abs_dip")	


                call to_history("n1",nt,"azi"); call to_history("n2",nx,"azi")
                call to_history("n3",ny,"azi")
		call to_history("d1",dt,"azi"); call to_history("d2",dx,"azi")
                call to_history("d3",dy,"azi")
		call to_history("o1",t0,"azi"); call to_history("o2",x0,"azi")
                call to_history("o3",y0,"azi")
                call to_history("esize",4,"azi")
		call sep_write(a1,"azi")	





end program Puck_3D_sm


















