module flat_mod
use fft_data_mod
contains
  subroutine expand_weight(weight,direction)
    use sep

    !  cause a weight to expand by one sample in a direction, either x(1) or y(2)
    ! weight is the input weight, direction is either 1 for the x direction and 2 for the y direction
    integer 	                          :: n1, n2, n3, i1,i2,i3,direction
    integer, dimension(:,:,:), allocatable    :: knownvec
    real,    dimension (:,:,:),   allocatable :: inwt,weight
    n1=size(weight(1:,1,1))
    n2=size(weight(1,1:,1))
    n3=size(weight(1,1,1:))

    write(0,*) n1,n2,n3


    allocate (inwt(n1,n2,n3))          

    inwt=weight
    ! outwt=inwt 

    !take each point in the cube
    do i3=1,n3
       do i2=1,n2
          do i1=1,n1      
             if (inwt(i1,i2,i3)==1) then  ! if already zero then get next sample
                if (direction==2) then ! if y direction
                   if (inwt(i1,i2,int(min(i3+1,n3)))==0) then
                      weight(i1,i2,i3)=0.
                   end if
                   weight(i1,i2,n3)=0.  ! set end to zero....dip cannot calculate dip at the last sample
                else  ! assume in x direction
                   if      (inwt(i1,int(min(i2+1,n2)),i3)==0) then
                      weight(i1,i2,i3)=0
                   end if
                   weight(i1,n2,i3)=0.  ! set end to zero....dip cannot calculate dip at the last sample
                end if

             end if



          end do
       end do
    end do




    deallocate (inwt)

  end subroutine expand_weight

  subroutine anti_alias(input,output)
    integer                                ::i2,i3
    integer                                ::it,i2_new,i3_new
    integer                                :: n1,n2,n3,n2_new,n3_new
    real                                   ::max_freq_n2,max_freq_n3,n2_upper,n3_upper,n2_lower,n3_lower
    real                                   ::ky,kz,cutout,notcut,cutratio
    real, allocatable        :: FrequRangeZ(:), FrequRangeY(:)
    real,dimension (:,:,:)                 :: input,output
    complex,dimension (:,:) ,allocatable   :: fft_cmplx
    type(fft_type)                         :: fft_2d_id

    n1 = size( input, 1) 
    n2 = size( input, 2)
    n3 = size( input, 3)

    n2_new=size(output,2)
    n3_new=size(output,3)

    ! INITIALIZE FFTW
    fft_2d_id  =init_fft_2d (n2 ,n3)  
    write(0,*) 'after fft'
    write(0,*) 'n1,n2,n3',n1,n2,n3,n2_new,n3_new
    allocate(fft_cmplx(n2,n3))
    allocate(FrequRangeY(n2), FrequRangeZ(n3))
    ! DEFINE FREQUENCY RANGES FOR EACH DIMENSION
    do i2=1,(n2/2+1)
       FrequRangeY(i2)=(1./n2)*(i2-1)
    end do
    do i2=n2/2+2,n2  
       FrequRangeY(i2)=-1*FrequRangeY(n2/2)+(1./n2)*(i2-n2/2-2)
    end do
    do i3=1,(n3/2+1)
       FrequRangeZ(i3)=(1./n3)*(i3-1)
    end do
    do i3=n3/2+2,n3  
       FrequRangeZ(i3)=-1*FrequRangeZ(n3/2)+(1./n3)*(i3-n3/2-2)
    end do
    write(0,*) FrequRangeZ
    max_freq_n2=(real(n2_new)/(2.*real(n2)))
    max_freq_n3=(real(n3_new)/(2.*real(n3)))
    write(0,*) 'max freq n2, n3',max_freq_n2,max_freq_n3
    n2_lower=max_freq_n2-0.1 !.1
    n2_upper=max_freq_n2+.2
    n3_lower=max_freq_n3-0.1!.1
    n3_upper=max_freq_n3+.2

    timeslice:   do it=1, n1  ! LOOP OVER TIMESLICES

notcut=0.
cutout=0.
cutratio=0.

       fft_cmplx=cmplx(input(it,1:,1:),0.)

       ! APPLY FFTW
       call fft_2d_data(fft_2d_id,FFTW_FORWARD,n2,n3,fft_cmplx)
       do i2=1,n2  
          do i3=1,n3
             ky=abs(FrequRangeY(i2))
             kz=abs(FrequRangeZ(i3)) 
if (ky<=n2_lower.and.kz<=n3_lower) notcut=notcut+abs(fft_cmplx(i2,i3))

             if (ky>n2_lower)then !.and.ky<n2_upper) then
                cutout=cutout+abs(fft_cmplx(i2,i3))*(1-(-.1)*(ky-n2_lower)) !(1.-exp(-10.*(ky-n2_lower)))
                fft_cmplx(i2,i3)=fft_cmplx(i2,i3)*((-.1)*(ky-n2_lower)) !exp(-10.*(ky-n2_lower))   !((-1/10)*(ky-n2_lower))
                ! else if  (ky>=n2_upper) then
                !    fft_cmplx(i2,i3)=fft_cmplx(i2,i3)*0.
             end if
             if (kz>n3_lower) then !.and.kz<n3_upper) then
                cutout=cutout+abs(fft_cmplx(i2,i3))*(1-(-.1)*(ky-n2_lower)) !(1.-exp(-10.*(kz-n3_lower)))
                fft_cmplx(i2,i3)=fft_cmplx(i2,i3)*((-.1)*(kz-n3_lower))!exp(-10.*(kz-n3_lower))  ! ((-1/10)*(kz-n3_lower))

                ! else if  (kz>=n3_upper) then
                !    fft_cmplx(i2,i3)=fft_cmplx(i2,i3)*0.
             end if
          end do
       end do
cutratio=(cutout+notcut)/cutout
write(0,*) 'cutratio',cutratio
       call fft_2d_data(fft_2d_id,FFTW_BACKWARD,n2,n3,fft_cmplx)
       do i2_new=1,n2_new
          do i3_new=1,n3_new
             output(it,i2_new,i3_new)=cutratio*real(fft_cmplx(nint(1.+real(i2_new-1.)*(real(n2)/real(n2_new))),nint(1.+(real(i3_new)-1.)*(real(n3)/real(n3_new)))))
          end do
       end do




    end do timeslice

    call clean_fft(fft_2d_id)

  end subroutine anti_alias
  end module flat_mod














